package core

import (
	"math"

	"git.akyoto.dev/cli/q/src/asm"
	"git.akyoto.dev/cli/q/src/ast"
	"git.akyoto.dev/cli/q/src/cpu"
	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/token"
)

// CompileAssignArray compiles an assign statement for array elements.
func (f *Function) CompileAssignArray(node *ast.Assign) error {
	left := node.Expression.Children[0]
	right := node.Expression.Children[1]

	name := left.Children[0].Token.Text(f.File.Bytes)
	variable := f.VariableByName(name)

	if variable == nil {
		return errors.New(&errors.UnknownIdentifier{Name: name}, f.File, left.Children[0].Token.Position)
	}

	defer f.UseVariable(variable)

	memory := asm.Memory{
		Base:           variable.Register,
		Offset:         0,
		OffsetRegister: cpu.Register(math.MaxUint8),
		Length:         byte(1),
	}

	index := left.Children[1]

	if index.Token.Kind == token.Number {
		offset, err := f.Number(index.Token)

		if err != nil {
			return err
		}

		memory.Offset = byte(offset)
	} else {
		_, indexRegister, isTemporary, err := f.Evaluate(index)

		if err != nil {
			return err
		}

		memory.OffsetRegister = indexRegister

		if isTemporary {
			defer f.FreeRegister(indexRegister)
		}
	}

	_, err := f.ExpressionToMemory(right, memory)
	return err
}
